@@ -2,6 +2,10 @@ from functools import reduce
 from operator import mod
 from random import randint
 
+import os
+
+Default_Libs_Path = "C:" + os.environ["homepath"] + "\\NumberScript\\Libs\\" if os.name == "nt" else os.environ["HOME"] + "/NumberScript/Libs/" 
+
 def Calculate(removed_code: str, variables: dict) -> int:
     """Calculator"""
     if "+" in removed_code:
@@ -162,7 +166,7 @@ class Interpreter():
         """Initialize the interpreter"""
         pass
 
-    def interpret(self, code: str, variables_dict: dict = {}, function_dict: dict = {}) -> str:
+    def interpret(self, code: str, variables_dict: dict = {}, function_dict: dict = {}, debug_mode: bool = False,library: bool = False) -> str:
         """Interprets the code"""
         code = code.split(" ")
         variables = variables_dict
@@ -171,11 +175,17 @@ class Interpreter():
             if code[j].startswith("0"):
                 variables = {}
             
-            if code[j].startswith("1"):
-                variables = {}
-                return
+            elif code[j].startswith("1"):
+                break
+            
+            elif code[j].startswith("#"):
+                recode = code[j].replace("#", "", 1)
+                file_location = f"{recode}.ns" if os.path.exists(f"{recode}.ns") else Default_Libs_Path + f"{recode}.ns"
+                module_code = open(f"{file_location}", "r").read()
+                module_code = module_code.replace("\n", " ")
+                module_args = self.interpret(Interpreter, module_code, variables, functions, library=True)
 
-            if code[j].startswith("2"):
+            elif code[j].startswith("2"):
                 if code[j][1:] in variables.keys():
                     for x in range(0, len(variables)):
                         var_names = list(variables.keys())
@@ -198,7 +208,7 @@ class Interpreter():
                 else:
                     print(code[j][1:])
 
-            if code[j].startswith("3"):
+            elif code[j].startswith("3"):
                 vars = code[j].replace("3", "", 1).split(":")
                 try:
                     var_name = vars[0]
@@ -239,20 +249,19 @@ class Interpreter():
                         return print("Error: Index Not A Number")
                     
                 variables[var_name] = var_content
-                
 
-            if code[j].startswith("^"):
+            elif code[j].startswith("^"):
                 removed_code = code[j].replace("^", "")
                 print(Calculate(removed_code, variables))
 
-            if code[j].startswith("%"):
+            elif code[j].startswith("%"):
                 continue
-            
-            if code[j].startswith("4"):
+
+            elif code[j].startswith("4"):
                 re_code = code[j].replace("4", "", 1)
                 print(Checker(re_code, variables))
-            
-            if code[j].startswith("?"):
+
+            elif code[j].startswith("?"):
                 re_code = code[j].replace("?", "", 1)
                 statements = re_code.split("|", 2)
                 if Checker(statements[0], variables):
@@ -268,41 +277,40 @@ class Interpreter():
                         false_statements = statements[2]
                     self.interpret(Interpreter, false_statements, variables)
 
-            if code[j] in variables.keys():
+            elif code[j] in variables.keys():
                 print(variables[code[j]])
 
-            if code[j].startswith("5"):
+            elif code[j].startswith("5"):
                 pass
 
-            if code[j].startswith("6"):
+            elif code[j].startswith("6"):
                 recode = code[j].replace("6", "", 1).split("\\", 2)
                 repeat_name = variables[recode[0]] if recode[0] in variables.keys() else recode[0]
                 repeat_times = int(variables[recode[1]]) if recode[1] in variables.keys() else int(recode[1])
                 repeat_code = recode[2].replace(";", " ")
                 variables[repeat_name] = 0
-                variabless = variables
+                loop_variables = variables
                 for x in range(0, repeat_times):
-                    self.interpret(Interpreter, repeat_code, variabless)
-                    variabless[repeat_name] += 1
+                    self.interpret(Interpreter, repeat_code, loop_variables)
+                    loop_variables[repeat_name] += 1
 
-            if code[j].startswith("~"):
+            elif code[j].startswith("~"):
                 recode = code[j].replace("~", "", 1)
-                input(recode)
+                input(recode if recode not in list(variables.keys()) else variables[recode])
 
-            if code[j].startswith("7"):
+            elif code[j].startswith("7"):
                 recode = code[j].replace("7", "", 1).split("$", 2)
                 function_name = recode[0]
-                function_parameters = recode[1].split(",")
-                function_code = recode[2].split("|")
+                function_parameters = recode[1].split(",") if "," in recode[1] else [recode[1]]
+                function_code = recode[2].split("$")
                 functions[function_name] = [function_parameters, function_code]
 
-            if code[j].startswith("*"):
+            elif code[j].startswith("*"):
                 recode = code[j].replace("*", "", 1).split("!", 1)
                 random_num = randint(recode[0], recode[1])
                 print(int(random_num))
 
-
-            if any(str(key) in code[j] for key in functions.keys()) and not code[j].startswith("7"):
+            elif any(str(key) in code[j] for key in functions.keys()) and not code[j].startswith("7"):
                 recode = code[j].split("$", 1)
                 func_name = functions[recode[0]]
                 func_params = recode[1].split(",")
@@ -329,4 +337,15 @@ class Interpreter():
                 for j in range(0, len(func_name[1])):
                     func_code += func_name[1][j] + " "
                 self.interpret(Interpreter, func_code, variables, functions)
+                return
+
+            else:
+                pass
+            
+            if debug_mode == True:
+                print(variables)
+                print(functions)
+
+        if library == True:
+            return [variables, functions]
                 
\ No newline at end of file
